@@ -1,8 +1,22 @@
-//#include "stdafx.h"
-#include "zlib/zlib.h"
 #include "bzip2/bzlib.h"
 #include "UnAlz.h"
-#include <errno.h>
+
+#ifdef _WIN32
+#	include "zlib/zlib.h"
+#else
+#	include <zlib.h>
+#endif
+
+
+// utime Լ ó
+#if defined(_WIN32) || defined(__CYGWIN__)
+#	include <time.h>
+#	include <sys/utime.h>
+#endif
+#ifdef __GNUC__
+#	include <time.h>
+#	include <utime.h>
+#endif
 
 // mkdir
 #ifdef _WIN32				
@@ -15,61 +29,52 @@
 #	include <iconv.h>
 #endif
 
-#ifdef __linux__			// __BYTE_ORDER  
+#if defined(__linux__) || defined(__GLIBC__) || defined(__GNU__) || defined(__APPLE__)
+#	include <errno.h>
+#endif
+
+#if defined(__NetBSD__)
+#	include <sys/param.h>	// __NetBSD_Version__
 #	include <errno.h>		// iconv.h  ʿ 
 #endif
 
+#ifdef _WIN32				// string conversion
+#	include <atlbase.h>
+#	include <atlconv.h>
+#endif
 
-#define swapint64(Data) (INT64) ( (((Data)&0x00000000000000FFLL) << 56) | (((Data)&0x000000000000FF00LL) << 40) | (((Data)&0x0000000000FF0000LL) << 24) | (((Data)&0x00000000FF000000LL) << 8)  | (((Data)&0x000000FF00000000LL) >> 8)  | (((Data)&0x0000FF0000000000LL) >> 24) | (((Data)&0x00FF000000000000LL) >> 40) | (((Data)&0xFF00000000000000LL) >> 56) )
+#ifdef _WIN32				// safe string ó
+#	include <strsafe.h>
+#endif
+
+
+// ENDIAN ó
+#ifdef _WIN32	// (L)
+#	define swapint64(a)	(UINT64) ( (((a)&0x00000000000000FFL) << 56) | (((a)&0x000000000000FF00L) << 40) | (((a)&0x0000000000FF0000L) << 24) | (((a)&0x00000000FF000000L) << 8)  | (((a)&0x000000FF00000000L) >> 8)  | (((a)&0x0000FF0000000000L) >> 24) | (((a)&0x00FF000000000000L) >> 40) | (((a)&0xFF00000000000000L) >> 56) )
+#else			// (LL)
+#	define swapint64(a)	(UINT64) ( (((a)&0x00000000000000FFLL) << 56) | (((a)&0x000000000000FF00LL) << 40) | (((a)&0x0000000000FF0000LL) << 24) | (((a)&0x00000000FF000000LL) << 8)  | (((a)&0x000000FF00000000LL) >> 8)  | (((a)&0x0000FF0000000000LL) >> 24) | (((a)&0x00FF000000000000LL) >> 40) | (((a)&0xFF00000000000000LL) >> 56) )
+#endif
 #define swapint32(a)    ((((a)&0xff)<<24)+(((a>>8)&0xff)<<16)+(((a>>16)&0xff)<<8)+(((a>>24)&0xff)))
 #define swapint16(a)    (((a)&0xff)<<8)+(((a>>8)&0xff))
 
-////////////////////////////////////////////////////////////////////////////
-//// byte-order : little to host                                        ////
-////////////////////////////////////////////////////////////////////////////
+typedef UINT16	(*_unalz_le16toh)(UINT16 a);
+typedef UINT32	(*_unalz_le32toh)(UINT32 a);
+typedef UINT64	(*_unalz_le64toh)(UINT64 a);
 
-#ifdef _WIN32		// little to little
-	inline UINT16	unalz_le16toh(UINT16 a){return a;}
-	inline UINT32	unalz_le32toh(UINT32 a){return a;}
-	inline UINT64	unalz_le64toh(UINT64 a){return a;}
-#endif
-
-#ifdef __FreeBSD__
-#	include <sys/endian.h>	
-	inline UINT16	unalz_le16toh(UINT16 a){return le16toh(a);}
-	inline UINT32	unalz_le32toh(UINT32 a){return le32toh(a);}
-	inline UINT64	unalz_le64toh(UINT64 a){return le64toh(a);}
-#endif
-
-#ifdef __APPLE__
-#	include <machine/byte_order.h>
-	inline UINT16   unalz_le16toh(UINT16 a){return NXSwapShort(a);} 
-	inline UINT32   unalz_le32toh(UINT32 a){return NXSwapLong(a);} 
-	inline UINT64   unalz_le64toh(UINT64 a){return NXSwapLongLong(a);} 
-#endif
-
-
-#ifdef __linux__	
-#	include <endian.h>
-#	if __BYTE_ORDER == __BIG_ENDIAN
-	inline UINT16	unalz_le16toh(UINT16 a){return swapint16(a);}
-	inline UINT32	unalz_le32toh(UINT32 a){return swapint32(a);}
-	inline UINT64	unalz_le64toh(UINT64 a){return swapint64(a);}
-#	else		// __LITTLE_ENDIAN
-	inline UINT16	unalz_le16toh(UINT16 a){return (a);}
-	inline UINT32	unalz_le32toh(UINT32 a){return (a);}
-	inline UINT64	unalz_le64toh(UINT64 a){return (a);}
-#	endif
-//#	include <asm/byteorder.h>
-//	inline UINT16	unalz_le16toh(UINT16 a){return le16_to_cpu(a);}
-//	inline UINT32	unalz_le32toh(UINT32 a){return le32_to_cpu(a);}
-//	inline UINT64	unalz_le64toh(UINT64 a){return le64_to_cpu(a);}
-#endif
+static _unalz_le16toh unalz_le16toh=NULL;
+static _unalz_le32toh unalz_le32toh=NULL;
+static _unalz_le64toh unalz_le64toh=NULL;
 
+static UINT16	le16tole(UINT16 a){return a;}
+static UINT32	le32tole(UINT32 a){return a;}
+static UINT64	le64tole(UINT64 a){return a;}
+static UINT16	le16tobe(UINT16 a){return swapint16(a);}
+static UINT32	le32tobe(UINT32 a){return swapint32(a);}
+static UINT64	le64tobe(UINT64 a){return swapint64(a);}
 
 
 #ifndef MAX_PATH
-#	define MAX_PATH 260
+#	define MAX_PATH 260*6			// ׳ .. ..
 #endif
 
 #ifdef _WIN32
@@ -80,13 +85,59 @@
 #	define PATHSEPC '/'
 #endif
 
+static time_t dosTime2TimeT(UINT32 dostime)   // from INFO-ZIP src
+{
+	struct tm t;         
+	t.tm_isdst = -1;     
+	t.tm_sec  = (((int)dostime) <<  1) & 0x3e;
+	t.tm_min  = (((int)dostime) >>  5) & 0x3f;
+	t.tm_hour = (((int)dostime) >> 11) & 0x1f;
+	t.tm_mday = (int)(dostime >> 16) & 0x1f;
+	t.tm_mon  = ((int)(dostime >> 21) & 0x0f) - 1;
+	t.tm_year = ((int)(dostime >> 25) & 0x7f) + 80;
+	return mktime(&t);
+}
+
+static BOOL IsBigEndian(void) 
+{
+   union {
+        short a;
+        char  b[2];
+   } endian;
+   
+   endian.a = 0x0102;
+   if(endian.b[0] == 0x02) return FALSE;
+   return TRUE;
+}
+
+
+#ifdef _WIN32
+#	define safe_sprintf	StringCbPrintfA
+#else
+#	define safe_sprintf	snprintf
+#endif
+
+
+// 64bit file handling support
+#if (_FILE_OFFSET_BITS==64)
+#	define unalz_fseek	fseeko
+#	define unalz_ftell	ftello
+#else
+#	define unalz_fseek	fseek
+#	define unalz_ftell	ftell
+#endif
 
 
 // error string table <- CUnAlz::ERR  
 static const char* errorstrtable[]=
 {
 	"no error",										// ERR_NOERR
-	"can't open file",								// ERR_CANT_OPEN_FILE
+	"general error",								// ERR_GENERAL
+	"can't open archive file",						// ERR_CANT_OPEN_FILE
+	"can't open dest file or path",					// ERR_CANT_OPEN_DEST_FILE
+//	"can't create dest path",						// ERR_CANT_CREATE_DEST_PATH
+	"corrupted file",								// ERR_CORRUPTED_FILE
+	"not alz file",									// ERR_NOT_ALZ_FILE
 	"can't read signature",							// ERR_CANT_READ_SIG
 	"can't read file",								// ERR_CANT_READ_FILE
 	"error at read header",							// ERR_AT_READ_HEADER
@@ -98,8 +149,11 @@ static const char* errorstrtable[]=
 	"invalid filecomment size",						// ERR_INVALID_FILECOMMENT_SIZE,
 	"cant' read header",							// ERR_CANT_READ_HEADER,
 	"memory allocation failed",						// ERR_MEM_ALLOC_FAILED,
-	"file read eror",								// ERR_FILE_READ_ERROR,
+	"file read error",								// ERR_FILE_READ_ERROR,
 	"inflate failed",								// ERR_INFLATE_FAILED,
+	"bzip2 decompress failed",						// ERR_BZIP2_FAILED,
+	"invalid file CRC",								// ERR_INVALID_FILE_CRC
+	"unknown compression method",					// ERR_UNKNOWN_COMPRESSION_METHOD
 									
 	"iconv-can't open iconv",						// ERR_ICONV_CANT_OPEN,
 	"iconv-invalid multisequence of characters",	// ERR_ICONV_INVALID_MULTISEQUENCE_OF_CHARACTERS,
@@ -107,13 +161,12 @@ static const char* errorstrtable[]=
 	"iconv-not enough space of buffer to convert",	// ERR_ICONV_NOT_ENOUGH_SPACE_OF_BUFFER_TO_CONVERT,
 	"iconv-etc",									// ERR_ICONV_ETC,
 	
-	"password not set",								//ERR_PASSWD_NOT_SET,
-	"invalid password",								//ERR_INVALID_PASSWD,
-	"User Aborted",					
+	"password was not set",							// ERR_PASSWD_NOT_SET,
+	"invalid password",								// ERR_INVALID_PASSWD,
+	"user aborted",					
 };
 
 
-
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 ///          ctor
 /// @date    2004-03-06  11:19:49
@@ -122,7 +175,7 @@ CUnAlz::CUnAlz()
 {
 	memset(m_files, 0, sizeof(m_files));
 	m_nErr = ERR_NOERR;
-	m_posCur = (FileList::iterator)NULL;
+	m_posCur = m_fileList.end();//(FileList::iterator)NULL;
 	m_pFuncCallBack = NULL;
 	m_pCallbackParam = NULL;
 	m_bHalt = FALSE;
@@ -133,17 +186,35 @@ CUnAlz::CUnAlz()
 	m_bIsEOF = FALSE;
 	m_bIsEncrypted = FALSE;
 	m_bIsDataDescr = FALSE;
+	m_bPipeMode = FALSE;
 
 #ifdef _UNALZ_ICONV
 
 #ifdef _UNALZ_UTF8
-	strcpy(m_szToCodepage, "UTF-8") ;		// ⺻ utf-8
+	safe_strcpy(m_szToCodepage, "UTF-8",UNALZ_LEN_CODEPAGE) ;		// ⺻ utf-8
 #else 
-	strcpy(m_szToCodepage, "CP949") ;		// ⺻ CP949
+	safe_strcpy(m_szToCodepage, "CP949",UNALZ_LEN_CODEPAGE) ;		// ⺻ CP949
 #endif // _UNALZ_UTF8
 
-	strcpy(m_szFromCodepage, "CP949");		// alz  949  
+	safe_strcpy(m_szFromCodepage, "CP949",UNALZ_LEN_CODEPAGE);		// alz  949  
 #endif // _UNALZ_ICONV
+
+	// check endian
+	if(unalz_le16toh==NULL)
+	{
+		if(IsBigEndian())
+		{
+			unalz_le16toh = le16tobe;
+			unalz_le32toh = le32tobe;
+			unalz_le64toh = le64tobe;
+		}
+		else
+		{
+			unalz_le16toh = le16tole;
+			unalz_le32toh = le32tole;
+			unalz_le64toh = le64tole;
+		}
+	}
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -173,8 +244,6 @@ void CUnAlz::SetCallback(_UnAlzCallback* pFunc, void* param)
 /// @return  
 /// @date    2004-03-06  11:03:59
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-#include <atlbase.h>
-#include <atlconv.h>
 BOOL CUnAlz::Open(LPCWSTR szPathName)
 {
 	USES_CONVERSION;
@@ -210,6 +279,8 @@ BOOL CUnAlz::Open(const char* szPathName)
 		return FALSE;
 	}
 
+	BOOL	bValidAlzHeader = FALSE;
+
 	// file м..
 	for(;;)
 	{
@@ -217,7 +288,7 @@ BOOL CUnAlz::Open(const char* szPathName)
 		BOOL		ret;
 
 		if(FEof()) break;
-		//int pos = ftell(m_fp);
+		//int pos = unalz_ftell(m_fp);
 		sig = ReadSignature();
 		if(sig==SIG_EOF)
 		{
@@ -225,17 +296,26 @@ BOOL CUnAlz::Open(const char* szPathName)
 		}
 		if(sig==SIG_ERROR)
 		{
+			if(bValidAlzHeader)
+				m_nErr = ERR_CORRUPTED_FILE;	// ջ 
+			else
+				m_nErr = ERR_NOT_ALZ_FILE;	// alz  ƴϴ.
 			return FALSE;						//  ..
 		}
 
-		if(sig==SIG_ALZ_FILE_HEADER) ret = ReadAlzFileHeader();
+		if(sig==SIG_ALZ_FILE_HEADER) 
+		{
+			ret = ReadAlzFileHeader();
+			bValidAlzHeader = TRUE;					// alz  ´.
+		}
 		else if(sig==SIG_LOCAL_FILE_HEADER) ret = ReadLocalFileheader();
 		else if(sig==SIG_CENTRAL_DIRECTORY_STRUCTURE) ret = ReadCentralDirectoryStructure();
 		else if(sig==SIG_ENDOF_CENTRAL_DIRECTORY_RECORD) ret = ReadEndofCentralDirectoryRecord();
 		else
 		{
-			// ̱ signature
+			// ̱ signature ?   ?
 			ASSERT(0);
+			m_nErr = ERR_CORRUPTED_FILE;
 			return FALSE;
 		}
 		
@@ -267,7 +347,7 @@ void CUnAlz::Close()
 		i->Clear();
 	}
 
-	m_posCur = (FileList::iterator)NULL;
+	m_posCur = m_fileList.end();//(FileList::iterator)NULL;
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -280,7 +360,6 @@ CUnAlz::SIGNATURE CUnAlz::ReadSignature()
 	UINT32	dwSig;
 	if(FRead(&dwSig, sizeof(dwSig))==FALSE)
 	{
-//int pos = ftell(m_fp);
 		if(FEof())
 			return SIG_EOF;
 		m_nErr = ERR_CANT_READ_SIG;
@@ -304,7 +383,6 @@ BOOL CUnAlz::ReadAlzFileHeader()
 		m_nErr = ERR_CANT_READ_FILE;
 		return FALSE;
 	}
-
 	return TRUE;
 }
 
@@ -315,7 +393,7 @@ BOOL CUnAlz::ReadAlzFileHeader()
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 BOOL CUnAlz::ReadLocalFileheader()
 {
-	SLocalFileHeader zipHeader;
+	SAlzLocalFileHeader zipHeader;
 	int ret;
 
 	ret = FRead(&(zipHeader.head), sizeof(zipHeader.head));
@@ -326,33 +404,33 @@ BOOL CUnAlz::ReadLocalFileheader()
 	}
 
 	// ALZ Ȯ..
-	if( (zipHeader.head.fileSizeByte & (SHORT)1) != 0){
-		m_bIsEncrypted = TRUE;
+	if( (zipHeader.head.fileDescriptor & (SHORT)1) != 0){
+		m_bIsEncrypted = TRUE;									// ϳ ȣ ɷ Ѵ.
 	}
-	if( (zipHeader.head.fileSizeByte & (SHORT)8) != 0){
+	if( (zipHeader.head.fileDescriptor & (SHORT)8) != 0){
 		m_bIsDataDescr = TRUE;
 	}
 	
-	int byteLen = zipHeader.head.fileSizeByte/0x10;
+	int byteLen = zipHeader.head.fileDescriptor/0x10;
 
 	if(byteLen)
 	{
 		FRead(&(zipHeader.compressionMethod),	sizeof(zipHeader.compressionMethod));
-		FRead(&(zipHeader.unknown3),			sizeof(zipHeader.unknown3));
-		FRead(&(zipHeader.maybeCRC),			sizeof(zipHeader.maybeCRC));
+		FRead(&(zipHeader.unknown),				sizeof(zipHeader.unknown));
+		FRead(&(zipHeader.fileCRC),				sizeof(zipHeader.fileCRC));
 
 		FRead(&(zipHeader.compressedSize), byteLen);
 		FRead(&(zipHeader.uncompressedSize),  byteLen);			//   .
 	}
 
 	// little to system 
+    zipHeader.fileCRC				=   unalz_le32toh(zipHeader.fileCRC);
     zipHeader.head.fileNameLength   =   unalz_le16toh(zipHeader.head.fileNameLength);
     zipHeader.compressedSize        =   unalz_le64toh(zipHeader.compressedSize);
     zipHeader.uncompressedSize      =   unalz_le64toh(zipHeader.uncompressedSize); 
-    zipHeader.maybeCRC			    =	unalz_le32toh(zipHeader.maybeCRC);
 
 	// FILE NAME
-	zipHeader.fileName = (char*)malloc(zipHeader.head.fileNameLength+1);
+	zipHeader.fileName = (char*)malloc(zipHeader.head.fileNameLength+sizeof(char));
 	if(zipHeader.fileName==NULL)
 	{
 		m_nErr = ERR_INVALID_FILENAME_LENGTH;
@@ -376,7 +454,7 @@ BOOL CUnAlz::ReadLocalFileheader()
 	size_t size;
 	char inbuf[ICONV_BUF_SIZE];
 	char outbuf[ICONV_BUF_SIZE];
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__CYGWIN__) ||  defined(__NetBSD__)
 	const char *inptr = inbuf;
 #else
 	char *inptr = inbuf;
@@ -415,7 +493,14 @@ BOOL CUnAlz::ReadLocalFileheader()
 		else 
 		{
 			outbuf[ICONV_BUF_SIZE-oleft] = 0;
-			strcpy(zipHeader.fileName, outbuf);
+			if(zipHeader.fileName) free(zipHeader.fileName);
+			zipHeader.fileName = strdup(outbuf);
+			if (zipHeader.fileName == NULL)
+			{
+				m_nErr = ERR_ICONV_ETC;
+				iconv_close(cd);
+				return FALSE;
+			}
 			// printf("\n  Converted File Name : %s", outbuf);
 		}
 		
@@ -439,7 +524,8 @@ BOOL CUnAlz::ReadLocalFileheader()
 	}
 	*/
 
-	if(IsEncrypted()) FRead(zipHeader.encChk, ENCR_HEADER_LEN);  // xf86
+	if(IsEncryptedFile(zipHeader.head.fileDescriptor)) 
+		FRead(zipHeader.encChk, ALZ_ENCR_HEADER_LEN);  // xf86
 
 	// SKIP FILE DATA
 	zipHeader.dwFileDataPos = FTell();						// data  ġ ϱ..
@@ -453,6 +539,7 @@ BOOL CUnAlz::ReadLocalFileheader()
 	}
 	*/
 
+	/*
 #ifdef _DEBUG
 	printf("NAME:%s COMPRESSED SIZE:%d UNCOMPRESSED SIZE:%d COMP METHOD:%d\n", 
 		zipHeader.fileName, 
@@ -461,6 +548,7 @@ BOOL CUnAlz::ReadLocalFileheader()
 		zipHeader.compressionMethod
 		);
 #endif
+	*/
 
 	//  Ͽ ߰Ѵ..
 	m_fileList.push_back(zipHeader);
@@ -517,7 +605,6 @@ BOOL CUnAlz::ReadCentralDirectoryStructure()
 		FRead(header.fileComment, 1, header.head.fileCommentLength, m_fp);
 		header.fileComment[header.head.fileCommentLength] = NULL;
 	}
-
 	*/
 
 	return TRUE;
@@ -570,11 +657,16 @@ BOOL CUnAlz::SetCurrentFile(const char* szFileName)
 		}
 	}
 
-	m_posCur = (FileList::iterator)NULL;
+	m_posCur = m_fileList.end();//(FileList::iterator)NULL;
 
 	return FALSE;
 }
 
+void CUnAlz::SetCurrentFile(FileList::iterator newPos)
+{
+	m_posCur = newPos;
+}
+
 #ifndef MAX_WBITS
 #  define MAX_WBITS   15 /* 32K LZ77 window */
 #endif
@@ -606,26 +698,43 @@ BOOL CUnAlz::ExtractCurrentFileToBuf(BYTE* pDestBuf, int nBufSize)
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 BOOL CUnAlz::ExtractCurrentFile(const char* szDestPathName, const char* szDestFileName)
 {
-	if(m_posCur==(FileList::iterator)NULL) {ASSERT(0); return FALSE;}
-
+	if(m_posCur==m_fileList.end()/*(FileList::iterator)NULL*/) {ASSERT(0); return FALSE;}
 	BOOL	ret=FALSE;
 
 	SExtractDest	dest;
-	char	szDestPathFileName[MAX_PATH];
+	char			szDestPathFileName[MAX_PATH];
 	
 	if(chkValidPassword() == FALSE) 
 	{
 		return FALSE;
 	}
+
+	if( szDestPathName==NULL||
+		strlen(szDestPathName) + (szDestFileName?strlen(szDestFileName):strlen(m_posCur->fileName))+1 > MAX_PATH
+	   )	// check buffer overflow
+	{
+		ASSERT(0);
+		m_nErr = ERR_GENERAL;
+		return FALSE;
+	}
 	
 	// θ
-	strcpy(szDestPathFileName, szDestPathName);
+	safe_strcpy(szDestPathFileName, szDestPathName, MAX_PATH);
 	if(szDestPathFileName[strlen(szDestPathFileName)]!=PATHSEPC)
-		strcat(szDestPathFileName, PATHSEP);
+		safe_strcat(szDestPathFileName, PATHSEP, MAX_PATH);
 
 	// ϸ
-	if(szDestFileName) strcat(szDestPathFileName, szDestFileName);
-	else strcat(szDestPathFileName, m_posCur->fileName);
+	if(szDestFileName) safe_strcat(szDestPathFileName, szDestFileName, MAX_PATH);
+	else safe_strcat(szDestPathFileName, m_posCur->fileName, MAX_PATH);
+
+	// ../../    Ȯ
+	if( strstr(szDestPathFileName, "../")||
+		strstr(szDestPathFileName, "..\\"))
+	{
+		ASSERT(0);
+		m_nErr = ERR_GENERAL;
+		return FALSE;
+	}
 
 #ifndef _WIN32 
 	{
@@ -640,32 +749,59 @@ BOOL CUnAlz::ExtractCurrentFile(const char* szDestPathName, const char* szDestFi
 
 	// Ǯ  (  )
 	dest.nType = ET_FILE;
-	dest.fp = fopen(szDestPathFileName, "wb");
 
+	if(m_bPipeMode)
+		dest.fp = stdout;				// pipe mode   stdout 
+	else
+		dest.fp = fopen(szDestPathFileName, "wb");
+
+	// Ÿ  ..
+	if(m_bPipeMode==FALSE && (m_posCur->head.fileAttribute) & ALZ_FILEATTR_DIRECTORY )
+	{
+//printf("digpath:%s\n", szDestPathFileName);
+		// ı
+		DigPath(szDestPathFileName);
+		return TRUE;
+//		m_nErr = ERR_CANT_CREATE_DEST_PATH;
+//		return FALSE;
+	}
+
+	//   н - θ ĺ
 	if(dest.fp==NULL) 
 	{
-		DigPath(szDestPathFileName);		// θ /   ..
+		DigPath(szDestPathFileName);
 		dest.fp = fopen(szDestPathFileName, "wb");
 	}
 
-#ifdef _WIN32
+	// ׷ Ͽ н.
 	if(dest.fp==NULL) 
 	{
-		ASSERT(0); 
+		//    
+		m_nErr = ERR_CANT_OPEN_DEST_FILE;
+//printf("dest pathfilename:%s\n",szDestPathFileName);
 		if(m_pFuncCallBack)
 		{
-			CHAR buf[1024];
-			sprintf(buf, "   : %s", szDestPathFileName);
-			m_pFuncCallBack(buf, 0,0,m_pCallbackParam, NULL);
+//			CHAR buf[1024];
+//			sprintf(buf, "   : %s", szDestPathFileName);
+//			m_pFuncCallBack(buf, 0,0,m_pCallbackParam, NULL);
 		}
 		return FALSE;
 	}
-#endif
+//#endif
+
 	// CALLBACK 
 	if(m_pFuncCallBack) m_pFuncCallBack(m_posCur->fileName, 0,m_posCur->uncompressedSize,m_pCallbackParam, NULL);
 
 	ret = ExtractTo(&dest);
-	if(dest.fp!=NULL)fclose(dest.fp);
+	if(dest.fp!=NULL)
+	{
+		fclose(dest.fp);
+		// file time setting - from unalz_wcx_01i.zip
+		utimbuf tmp;
+		tmp.actime = 0;													//   Ÿ
+		tmp.modtime = dosTime2TimeT(m_posCur->head.fileTimeDate);		//  ڸ ( ڴ  ٲ?)
+		utime(m_posCur->fileName, &tmp);
+	}
 	return ret;
 }
 
@@ -691,9 +827,13 @@ BOOL CUnAlz::ExtractTo(SExtractDest* dest)
 	{
 		ret = ExtractDeflate2(dest, *m_posCur);			// deflate
 	}
-	else
+	else	// COMP_UNKNOWN
 	{
-		ASSERT(0);										// ο  ???
+		// alzip 5.6  ߰ (5.5  Ǯ Ѵ.  5.51  Ǭ ) 
+		//        Ȯ   .
+		//       ʴ´. (Ÿ?)     .
+		m_nErr = ERR_UNKNOWN_COMPRESSION_METHOD;
+		ASSERT(0);										
 		ret = FALSE;
 	}
 	return ret;
@@ -707,7 +847,7 @@ BOOL CUnAlz::ExtractTo(SExtractDest* dest)
 /// @date    2004-03-06  11:09:17
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 /*
-BOOL CUnAlz::ExtractDeflate(FILE* fp, SLocalFileHeader& file)
+BOOL CUnAlz::ExtractDeflate(FILE* fp, SAlzLocalFileHeader& file)
 {
 	z_stream	stream;
 	BYTE*		pInBuf=NULL;
@@ -800,9 +940,9 @@ BOOL CUnAlz::ExtractAll(const char* szDestPathName)
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 BOOL CUnAlz::DigPath(const char* szPathName)
 {
-	char* dup = strdup(szPathName);
-	char seps[]   = "/\\";
-	char *token;
+	char*	dup = strdup(szPathName);
+	char	seps[]   = "/\\";
+	char*	token;
 	char	path[MAX_PATH] = {0};
 	char*	last;
 
@@ -818,15 +958,22 @@ BOOL CUnAlz::DigPath(const char* szPathName)
 		last --;
 	}
 
+	
 	token = strtok( dup, seps );
 	while( token != NULL )
 	{
-		if(strlen(path)==0) 
-			strcpy(path, token);
+		if(strlen(path)==0)
+		{
+			if(szPathName[0]=='/')			// is absolute path ?
+				safe_strcpy(path,"/", MAX_PATH);
+			else if(szPathName[0]=='\\' && szPathName[1]=='\\')	// network drive ?
+				safe_strcpy(path,"\\\\", MAX_PATH);
+			safe_strcat(path, token, MAX_PATH);
+		}
 		else
 		{
-			strcat(path, PATHSEP);
-			strcat(path, token);
+			safe_strcat(path, PATHSEP,MAX_PATH);
+			safe_strcat(path, token,MAX_PATH);
 		}
 
 		if(IsFolder(path)==FALSE)
@@ -835,6 +982,7 @@ BOOL CUnAlz::DigPath(const char* szPathName)
 #else
 			mkdir(path,  S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
 #endif
+//printf("path:%s\n", path);
 		token = strtok( NULL, seps );
 	}
 
@@ -849,7 +997,7 @@ BOOL CUnAlz::DigPath(const char* szPathName)
 /// @return  
 /// @date    2004-03-06  11:03:26
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-BOOL CUnAlz::IsFolder(LPCSTR szPathName)
+BOOL CUnAlz::IsFolder(const CHAR* szPathName)
 {
 #ifdef _WIN32
 	UINT32 dwRet;
@@ -886,6 +1034,7 @@ int	CUnAlz::WriteToDest(SExtractDest* dest, BYTE* buf, int nSize)
 	}
 	else if(dest->nType==ET_MEM)
 	{
+		if(dest->buf==NULL) return nSize;			//  NULL ̴... Ǫ ô Ѵ..
 		if(dest->bufpos+nSize >dest->bufsize)		// .. ۰ ƴ.
 		{
 			ASSERT(0);
@@ -910,7 +1059,7 @@ int	CUnAlz::WriteToDest(SExtractDest* dest, BYTE* buf, int nSize)
 #define BZIP2_TAIL_SIZE		10		//  4+5 .?
 BYTE bzip2Header[BZIP2_HEADER_SIZE] = {0x42, 0x5a, 0x68, 0x39, 0x31, 0x41, 0x59, 0x26, 0x53, 0x59};
 
-BOOL CUnAlz::ExtractBzip2_bak(FILE* fp, SLocalFileHeader& file)
+BOOL CUnAlz::ExtractBzip2_bak(FILE* fp, SAlzLocalFileHeader& file)
 {
 	bz_stream	stream;
 	BYTE*		pInBuf=NULL;
@@ -1003,21 +1152,26 @@ END :
 /// @date    2004-03-06  11:10:53
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 #define BUF_LEN		(4096*2)
-BOOL CUnAlz::ExtractRawfile(SExtractDest* dest, SLocalFileHeader& file)
+BOOL CUnAlz::ExtractRawfile(SExtractDest* dest, SAlzLocalFileHeader& file)
 {
-	BOOL	ret = FALSE;
-	BYTE	buf[BUF_LEN];
-	INT64	read;
-	INT64	sizeToRead;
-	INT64   bufLen = BUF_LEN;
-	INT64	nWritten = 0;
-	BOOL	bHalt = FALSE;
+	BOOL		ret = FALSE;
+	BYTE		buf[BUF_LEN];
+	INT64		read;
+	INT64		sizeToRead;
+	INT64		bufLen = BUF_LEN;
+	INT64		nWritten = 0;
+	BOOL		bHalt = FALSE;
+	BOOL		bIsEncrypted = 	IsEncryptedFile();		// ȣɸ ΰ?
+	UINT32		dwCRC32= 0;
+
+
 
 	// ġ .
 	FSeek(file.dwFileDataPos);
 
 	sizeToRead = file.compressedSize;			//  ũ.
 
+	m_nErr = ERR_NOERR;
 	while(sizeToRead)
 	{
 		read = min(sizeToRead, bufLen);
@@ -1025,8 +1179,11 @@ BOOL CUnAlz::ExtractRawfile(SExtractDest* dest, SLocalFileHeader& file)
 		{
 			break;
 		}
-		
-		CryptDecodeBuffer((int)read, (CHAR *)buf); // xf86
+
+		if(bIsEncrypted)
+			DecryptingData((int)read, buf); // xf86
+
+		dwCRC32 = crc32(dwCRC32, buf, (UINT)(read));
 
 		WriteToDest(dest, buf, (int)read);
 		//fwrite(buf, read, 1, fp);
@@ -1046,7 +1203,20 @@ BOOL CUnAlz::ExtractRawfile(SExtractDest* dest, SLocalFileHeader& file)
 	}
 
 	m_bHalt = bHalt;
-	ret = TRUE;
+
+	if(m_nErr==ERR_NOERR)			//   Ǯ.. CRC ˻ϱ..
+	{
+		if(file.fileCRC==dwCRC32)
+		{
+			ret = TRUE;
+		}
+		else
+		{
+			m_nErr = ERR_INVALID_FILE_CRC;
+		}
+	}
+
+
 	return ret;
 }
 
@@ -1058,16 +1228,18 @@ BOOL CUnAlz::ExtractRawfile(SExtractDest* dest, SLocalFileHeader& file)
 /// @date    2004-03-01  5:47:36
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 #define BZIP2_EXTRACT_BUF_SIZE	0x2000
-BOOL CUnAlz::ExtractBzip2(SExtractDest* dest, SLocalFileHeader& file)
+BOOL CUnAlz::ExtractBzip2(SExtractDest* dest, SAlzLocalFileHeader& file)
 {
-	BZFILE	*bzfp = NULL;
-	int		smallMode = 0;
-	int		verbosity = 1;
-	int		bzerr;
-	INT64	len;
-	char	buff[BZIP2_EXTRACT_BUF_SIZE];
-	INT64	nWritten = 0;
-	BOOL	bHalt = FALSE;
+	BZFILE		*bzfp = NULL;
+	int			smallMode = 0;
+	int			verbosity = 1;
+	int			bzerr;
+	INT64		len;
+	BYTE		buff[BZIP2_EXTRACT_BUF_SIZE];
+	INT64		nWritten = 0;
+	BOOL		bHalt = FALSE;
+	UINT32		dwCRC32= 0;
+	BOOL		ret = FALSE;
 
 	FSeek(file.dwFileDataPos);
 
@@ -1075,11 +1247,15 @@ BOOL CUnAlz::ExtractBzip2(SExtractDest* dest, SLocalFileHeader& file)
 
 	if(bzfp==NULL){ASSERT(0); return FALSE;}
 
+	m_nErr = ERR_NOERR;
 	while((len=BZ2_bzread(bzfp,buff,BZIP2_EXTRACT_BUF_SIZE))>0)
 	{
 		WriteToDest(dest, (BYTE*)buff, (int)len);
 		//fwrite(buff,1,len,fp_w);
 
+		dwCRC32 = crc32(dwCRC32,buff, (UINT)(len));
+
+
 		nWritten+=len;
 
 		// progress callback
@@ -1093,10 +1269,28 @@ BOOL CUnAlz::ExtractBzip2(SExtractDest* dest, SLocalFileHeader& file)
 		}
 	}
 
+	if(len<0)			//  Ȳ..
+	{
+		m_nErr = ERR_INFLATE_FAILED;
+	}
+
 	BZ2_bzReadClose( &bzerr, bzfp);
 
 	m_bHalt = bHalt;
 
+	if(m_nErr==ERR_NOERR)			//   Ǯ.. CRC ˻ϱ..
+	{
+		if(file.fileCRC==dwCRC32)
+		{
+			ret = TRUE;
+		}
+		else
+		{
+			m_nErr = ERR_INVALID_FILE_CRC;
+		}
+	}
+
+
 	/*		
 	// FILE*  Ұ ϴ ڵ. - Ƽ   ..
 	BZFILE	*bzfp = NULL;
@@ -1135,7 +1329,7 @@ BOOL CUnAlz::ExtractBzip2(SExtractDest* dest, SLocalFileHeader& file)
 	m_bHalt = bHalt;
 	*/
 
-	return TRUE;
+	return ret;
 }
 
 
@@ -1153,7 +1347,7 @@ BOOL CUnAlz::ExtractBzip2(SExtractDest* dest, SLocalFileHeader& file)
 /// @return  
 /// @date    2004-03-06  11:11:36
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-BOOL CUnAlz::ExtractDeflate2(SExtractDest* dest, SLocalFileHeader& file)
+BOOL CUnAlz::ExtractDeflate2(SExtractDest* dest, SAlzLocalFileHeader& file)
 {
 	z_stream	stream;
 	BYTE		pInBuf[IN_BUF_SIZE];
@@ -1169,6 +1363,7 @@ BOOL CUnAlz::ExtractDeflate2(SExtractDest* dest, SLocalFileHeader& file)
 	UINT		iRead = 0;
 	INT64	nWritten = 0;
 	BOOL	bHalt = FALSE;
+	BOOL	bIsEncrypted = 	IsEncryptedFile();		// ȣɸ ΰ?
 
 	memset(&stream, 0, sizeof(stream));
 
@@ -1182,6 +1377,7 @@ BOOL CUnAlz::ExtractDeflate2(SExtractDest* dest, SLocalFileHeader& file)
 	stream.next_out = pOutBuf;
 	stream.avail_out = OUT_BUF_SIZE;
 
+	m_nErr = ERR_NOERR;
 	while(stream.avail_out>0)
 	{
 		if(stream.avail_in==0 && nRestReadCompressed>0)
@@ -1198,8 +1394,11 @@ BOOL CUnAlz::ExtractDeflate2(SExtractDest* dest, SLocalFileHeader& file)
 				m_nErr = ERR_CANT_READ_FILE;
 				goto END;
 			}
+			
+			if(bIsEncrypted)
+				DecryptingData(uReadThis, pInBuf); // xf86
 
-			CryptDecodeBuffer(uReadThis, (CHAR *)pInBuf); // xf86
+//			dwCRC32 = crc32(dwCRC32,pInBuf, (UINT)(uReadThis));
 			
 			nRestReadCompressed -= uReadThis;
 			stream.next_in = pInBuf;
@@ -1238,6 +1437,7 @@ BOOL CUnAlz::ExtractDeflate2(SExtractDest* dest, SLocalFileHeader& file)
 			m_pFuncCallBack(NULL, nWritten, file.uncompressedSize, m_pCallbackParam, &bHalt);
 			if(bHalt) 
 			{
+				m_nErr = ERR_USER_ABORTED;
 				break;
 			}
 		}
@@ -1251,10 +1451,21 @@ BOOL CUnAlz::ExtractDeflate2(SExtractDest* dest, SLocalFileHeader& file)
             goto END;
 		}
 	}
-
 	m_bHalt = bHalt;
 
-	ret = TRUE;
+
+	if(m_nErr==ERR_NOERR)			//   Ǯ.. CRC ˻ϱ..
+	{
+		if(file.fileCRC==dwCRC32)
+		{
+			ret = TRUE;
+		}
+		else
+		{
+			m_nErr = ERR_INVALID_FILE_CRC;
+		}
+	}
+
 
 END :
 	inflateEnd(&stream);
@@ -1282,7 +1493,9 @@ BOOL CUnAlz::FOpen(const char* szPathName)
 	m_bIsEOF = FALSE;
 	for(i=0;i<MAX_FILES;i++)						// aa.alz ϸ  aa.a00 aa.a01 aa.a02 .. 
 	{
-		if(i>0) sprintf(temp+nLen-3, "%c%02d", (i-1)/100+'a', (i-1)%100);
+		if(i>0) 
+			safe_sprintf(temp+nLen-3, 4, "%c%02d", (i-1)/100+'a', (i-1)%100);
+
 #ifdef _WIN32
 		m_files[i].fp = CreateFileA(temp, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
 		if(m_files[i].fp==INVALID_HANDLE_VALUE) break;
@@ -1291,9 +1504,9 @@ BOOL CUnAlz::FOpen(const char* szPathName)
 		m_files[i].fp = fopen(temp, "rb");
 		if(m_files[i].fp==NULL) break;
 		dwFileSizeHigh=0;
-		fseek(m_files[i].fp,0,SEEK_END);		
-		nFileSizeLow=ftell(m_files[i].fp);
-		fseek(m_files[i].fp,0,SEEK_SET);
+		unalz_fseek(m_files[i].fp,0,SEEK_END);		
+		nFileSizeLow=unalz_ftell(m_files[i].fp);
+		unalz_fseek(m_files[i].fp,0,SEEK_SET);
 #endif
 		m_nFileCount++;
 		m_files[i].nFileSize = ((INT64)nFileSizeLow) + (((INT64)dwFileSizeHigh)<<32);
@@ -1326,6 +1539,7 @@ void CUnAlz::FClose()
 	m_nVirtualFilePos = 0;
 	m_nCurFilePos = 0;
 	m_bIsEOF = FALSE;
+
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -1380,7 +1594,7 @@ BOOL CUnAlz::FSeek(INT64 offset)
 #ifdef _WIN32
 			SetFilePointer(m_files[i].fp, LONG(m_nCurFilePos), &remainHigh, FILE_BEGIN);
 #else
-			fseek(m_files[i].fp, m_nCurFilePos, SEEK_SET);
+			unalz_fseek(m_files[i].fp, m_nCurFilePos, SEEK_SET);
 #endif
 			return TRUE;
 		}
@@ -1424,6 +1638,7 @@ BOOL CUnAlz::FRead(void* buffer, UINT32 nBytesToRead, int* pTotRead )
 		{
 			m_bIsEOF = TRUE;return FALSE;
 		}
+
 #else
 		nNumOfBytesRead = fread(((BYTE*)buffer)+dwTotRead, 1,dwRead ,m_files[m_nCurFile].fp);
 		if(nNumOfBytesRead<=0) 
@@ -1463,7 +1678,7 @@ BOOL CUnAlz::FRead(void* buffer, UINT32 nBytesToRead, int* pTotRead )
 #ifdef _WIN32
 			SetFilePointer(m_files[m_nCurFile].fp, (int)m_nCurFilePos, NULL, FILE_BEGIN);
 #else
-			fseek(m_files[m_nCurFile].fp, m_nCurFilePos, SEEK_SET);
+			unalz_fseek(m_files[m_nCurFile].fp, m_nCurFilePos, SEEK_SET);
 #endif
 		}
 		else
@@ -1489,21 +1704,22 @@ const char* CUnAlz::LastErrToStr(ERR nERR)
 // by xf86
 BOOL	CUnAlz::chkValidPassword()
 {
-	if (IsEncrypted()) {
-		if (getPasswordLen() == 0){
-			m_nErr = ERR_PASSWD_NOT_SET;
-			return FALSE;
-		}
-		CryptInitKeys();
-		if(CryptCheck(m_posCur->encChk) == FALSE){
-			m_nErr = ERR_INVALID_PASSWD;
-			return FALSE;
-		}
+	if(IsEncryptedFile()==FALSE) {return TRUE;}
+
+	if (getPasswordLen() == 0){
+		m_nErr = ERR_PASSWD_NOT_SET;
+		return FALSE;
+	}
+	InitCryptKeys(m_szPasswd);
+	if(CryptCheck(m_posCur->encChk) == FALSE){
+		m_nErr = ERR_INVALID_PASSWD;
+		return FALSE;
 	}
 	return TRUE;
 }
 
 
+/*
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 //	from CZipArchive
 //	Copyright (C) 2000 - 2004 Tadeusz Dracz
@@ -1512,7 +1728,6 @@ BOOL	CUnAlz::chkValidPassword()
 //
 //	it's under GPL.
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-
 void CUnAlz::CryptDecodeBuffer(UINT32 uCount, CHAR *buf)
 {
 	if (IsEncrypted())
@@ -1542,7 +1757,7 @@ void CUnAlz::CryptUpdateKeys(CHAR c)
 BOOL CUnAlz::CryptCheck(CHAR *buf)
 {
 	CHAR b = 0;
-	for (int i = 0; i < ENCR_HEADER_LEN; i++)
+	for (int i = 0; i < ALZ_ENCR_HEADER_LEN; i++)
 	{
 		b = buf[i]; 
 		CryptDecode((CHAR&)b);
@@ -1571,4 +1786,188 @@ UINT32 CUnAlz::CryptCRC32(UINT32 l, CHAR c)
 	const ULONG *CRC_TABLE = get_crc_table();
 	return CRC_TABLE[(l ^ c) & 0xff] ^ (l >> 8);
 }
+*/
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+///          ȣɸ  
+/// @param   fileDescriptor  
+/// @return  
+/// @date    2004-11-27  11:25:32
+////////////////////////////////////////////////////////////////////////////////////////////////////
+BOOL CUnAlz::IsEncryptedFile(BYTE fileDescriptor)
+{
+	return fileDescriptor&0x01;
+}
+BOOL CUnAlz::IsEncryptedFile()
+{
+	return m_posCur->head.fileDescriptor&0x01;
+}
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+///          ȣ Ű ʱȭ
+/// @param   szPassword  
+/// @return  
+/// @date    2004-11-27  11:04:01
+////////////////////////////////////////////////////////////////////////////////////////////////////
+void CUnAlz::InitCryptKeys(const CHAR* szPassword)
+{
+	m_key[0] = 305419896;
+	m_key[1] = 591751049;
+	m_key[2] = 878082192;
+
+	int i;
+	for(i=0;i<(int)strlen(szPassword);i++)
+	{
+		UpdateKeys(szPassword[i]);
+	}
+}
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+///          Ÿ Ű Ʈϱ
+/// @param   c  
+/// @return  
+/// @date    2004-11-27  11:04:09
+////////////////////////////////////////////////////////////////////////////////////////////////////
+void CUnAlz::UpdateKeys(BYTE c)
+{
+	m_key[0] = CRC32(m_key[0], c);
+	m_key[1] = m_key[1]+(m_key[0]&0x000000ff);
+	m_key[1] = m_key[1]*134775813+1;
+	m_key[2] = CRC32(m_key[2],m_key[1]>>24);
+}
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+///          ȣ ´  üũϱ 
+/// @param   buf  
+/// @return  
+/// @date    2004-11-27  11:04:24
+////////////////////////////////////////////////////////////////////////////////////////////////////
+BOOL CUnAlz::CryptCheck(const BYTE* buf)
+{
+	int i;
+	BYTE c;
+	BYTE temp[ALZ_ENCR_HEADER_LEN];
+
+	memcpy(temp, buf, ALZ_ENCR_HEADER_LEN);		// ӽ .
+
+	for(i=0;i<ALZ_ENCR_HEADER_LEN;i++)
+	{
+		c = temp[i] ^ DecryptByte();
+		UpdateKeys(c);
+		temp[i] = c;
+	}
+
+	if (IsDataDescr()) // Data descriptor present
+		return (m_posCur->head.fileTimeDate >> 8) == c;
+	else
+		return ( ((m_posCur->fileCRC)>>24) ) == c;		//  crc  ֻ byte 
+}
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+///          Ű Ÿ 
+/// @return  
+/// @date    2004-11-27  11:05:36
+////////////////////////////////////////////////////////////////////////////////////////////////////
+BYTE CUnAlz::DecryptByte()
+{
+	UINT16 temp;
+	temp = m_key[2] | 2;
+	return (temp * (temp^1))>>8;
+}
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+///          Ÿ  Ǯ
+/// @param   nSize  
+/// @param   data  
+/// @return  
+/// @date    2004-11-27  11:03:30
+////////////////////////////////////////////////////////////////////////////////////////////////////
+void CUnAlz::DecryptingData(int nSize, BYTE* data)
+{
+	BYTE* p = data;
+	BYTE temp;
+
+	while(nSize)
+	{
+		temp = *p ^ DecryptByte();
+		UpdateKeys(temp);
+		*p = temp;
+		nSize--;
+		p++;
+	}
+}
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+///          CRC ̺ 
+/// @param   l  
+/// @param   c  
+/// @return  
+/// @date    2004-11-27  11:14:16
+////////////////////////////////////////////////////////////////////////////////////////////////////
+UINT32 CUnAlz::CRC32(UINT32 l, BYTE c)
+{
+	const unsigned long *CRC_TABLE = get_crc_table();
+	return CRC_TABLE[(l ^ c) & 0xff] ^ (l >> 8);
+}
+
+void CUnAlz::SetPassword(char *passwd) 
+{ 
+	if(strlen(passwd) == 0) return; 
+	safe_strcpy(m_szPasswd, passwd, UNALZ_LEN_PASSWORD); 
+}
+
+#ifdef _UNALZ_ICONV
+void CUnAlz::SetDestCodepage(const char* szToCodepage)
+{
+	safe_strcpy(m_szToCodepage, szToCodepage, UNALZ_LEN_CODEPAGE); 
+}
+#endif
+
+
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+///          ڿ ó Լ
+/// @param   l  
+/// @param   c  
+/// @return  
+/// @date    2007-02
+////////////////////////////////////////////////////////////////////////////////////////////////////
+unsigned int CUnAlz::_strlcpy (char *dest, const char *src, unsigned int size)
+{
+	register unsigned int i = 0;
+	if (size > 0) {
+	size--;
+	for (i=0; size > 0 && src[i] != '\0'; ++i, size--)
+		dest[i] = src[i];
+	dest[i] = '\0';
+	}
+	while (src[i++]);
+	return i;
+}
+unsigned int CUnAlz::_strlcat (char *dest, const char *src, unsigned int size)
+{
+	register char *d = dest;
+	for (; size > 0 && *d != '\0'; size--, d++);
+	return (d - dest) + _strlcpy(d, src, size);
+}
+
+//  strcpy
+void CUnAlz::safe_strcpy(char* dst, const char* src, size_t dst_size)
+{
+#ifdef _WIN32
+	lstrcpynA(dst, src, dst_size);
+#else
+	_strlcpy(dst, src, dst_size);
+#endif
+}
+
+void CUnAlz::safe_strcat(char* dst, const char* src, size_t dst_size)
+{
+#ifdef _WIN32
+	StringCchCatExA(dst, dst_size, src, NULL, NULL, STRSAFE_FILL_BEHIND_NULL);
+	//lstrcatA(dst, src);			// not safe!!
+#else
+	_strlcat(dst, src, dst_size);
+#endif
+}
 
